Function LoadSprites()
	explosion_sprite=LoadSprite( "explosion.bmp" )
	HideEntity explosion_sprite

	projectile_sprite=LoadSprite( "projectile.bmp" )
	ScaleSprite projectile_sprite,3,3
	EntityRadius projectile_sprite,1.5
	EntityType projectile_sprite,coll_projectile
	HideEntity projectile_sprite
End Function

Function UpdateProjectiles()
	For p.Projectile=Each Projectile
		UpdateProjectile( p )
	Next
	For e.Explosion=Each Explosion
		UpdateExplosion( e )
	Next
End Function

Function CreateProjectile.Projectile( cam1 )
	p.Projectile=New Projectile
	p\time_out=150
	p\sprite=CopyEntity( projectile_sprite, cam1 )
	EntityParent p\sprite,0
	shootChannel = PlaySound (shoot)
	Return p
End Function

Function UpdateProjectile( p.Projectile )
	If CountCollisions( p\sprite )
		If EntityCollided( p\sprite,coll_dogs )
			For k=1 To CountCollisions( p\sprite )
				doghit=CollisionEntity( p\sprite,k )
				If GetEntityType( doghit )=coll_dogs
					Exit
				EndIf
			Next
			boomChannel = PlaySound (boom)
			CreateExplosion( p )
			FreeEntity p\sprite
			Delete p
			Return
		EndIf
	EndIf
	p\time_out=p\time_out-1
	If p\time_out=0
		FreeEntity p\sprite
		Delete p
		Return
	EndIf
	MoveEntity p\sprite,0,0,2
End Function

Function CreateExplosion.Explosion( p.Projectile )
	e.Explosion=New Explosion
	e\alpha=-90
	e\sprite=CopyEntity( explosion_sprite,p\sprite )
	EntityParent e\sprite,0
	Return e
End Function

Function UpdateExplosion( e.Explosion )
	If e\alpha<270
		ez#=Sin(e\alpha)*5+5
		ScaleSprite e\sprite,ez,ez
		e\alpha=e\alpha+15
	Else
		FreeEntity e\sprite
		Delete e
		For hit.dog = Each dog
			If hit\ID = doghit Delete hit : FreeEntity doghit
		Next
	EndIf
End Function

Function LoadTrees(file$)
	tree=LoadMesh(file$)
	ScaleMesh tree,.1,.1,.1
	For n=1 To 1000
		tree_copy=CopyEntity(tree)
		ScaleEntity tree_copy,Rand(1,4),Rand(1,4),Rand(1,4)
		x=Rand(-2000,2000)	:z=Rand(-2000,2000)
		PositionEntity tree_copy, x,TerrainY(land,x,0,z)-5,z
		RotateEntity tree_copy,0,Rand(1,180),0
		EntityAutoFade tree_copy,500,1000
		EntityRadius tree_copy,10
		EntityType tree_copy,coll_trees
	Next
	FreeEntity tree 	 
End Function

Function LoadDogs(file$)
	model=LoadAnimMesh(file$)
	ExtractAnimSeq model,2,14
	ScaleEntity model,.2,.2,.2
	For c=1 To 50
		Pack.dog = New dog
		Pack\ID = CopyEntity(model)
		x=Rand(-2000,2000):z=Rand(-2000,2000)
		PositionEntity Pack\ID, x,TerrainY(land,x,0,z),z
		Animate Pack\ID,1,.3,1
		EntityAutoFade Pack\ID,500,1000
		EntityType Pack\ID,coll_dogs
		EntityRadius Pack\ID,5
	Next
	FreeEntity model
End Function

Function MoveDogs(obj)
	For moveall.dog = Each dog
		PointEntity moveall\ID,obj
		MoveEntity moveall\ID,0,0,.5
		ex#=EntityX(moveall\ID):ez#=EntityZ(moveall\ID)
		PositionEntity moveall\ID,ex,TerrainY( land,ex,0,ez ),ez
	Next
End Function

Function LoadLights()
	Read numlights
	For n=1 To numlights
		Read x,z,R,G,B,range
		light=CreateLight(2)
		LightColor light, R,G,B
		LightRange light, range
		PositionEntity light, x,TerrainY(land,x,0,z)+50,z
	Next	 
End Function

Function LoadMusic(files$)
	bgsound = LoadSound (files$)
	LoopSound bgsound
	SoundVolume bgsound,.5 
	PlaySound bgsound
End Function

Function LoadSFX()
	shoot=LoadSound( "shoot.wav" )
	SoundVolume shoot, .9
	boom=LoadSound( "boom.wav" )
	SoundVolume boom, .5
	run = LoadSound ("gravel.wav")
	SoundVolume run, 0
	LoopSound run
	runChannel = PlaySound (run)
	wade = LoadSound ("water.wav")
	SoundVolume wade, 0
	LoopSound wade
	wadeChannel = PlaySound (wade)
End Function

Function object_key_control( obj )
	If KeyHit(57)	;spacebar = fire
		CreateProjectile( cam1 )
	EndIf
	walksnd=False:wadesnd=False
	If KeyDown( 200 )=True And EntityY(obj)>waterlevel+4 Then MoveEntity obj,0,0,1:walksnd=True 
	If KeyDown( 208 )=True And EntityY(obj)>waterlevel+4 Then MoveEntity obj,0,0,-1:walksnd=True
	If KeyDown( 200 )=True And EntityY(obj)<=waterlevel+4 Then MoveEntity obj,0,0,1:wadesnd=True 
	If KeyDown( 208 )=True And EntityY(obj)<=waterlevel+4 Then MoveEntity obj,0,0,-1:wadesnd=True
	If KeyDown( 203 )=True Then TurnEntity obj,0,2,0
	If KeyDown( 205 )=True Then TurnEntity obj,0,-2,0
	ex#=EntityX(obj):ez#=EntityZ(obj)
	PositionEntity obj,ex,TerrainY( land,ex,0,ez )+5,ez
	If walksnd Then run_vol#=1 Else run_vol#=0
	If wadesnd Then wade_vol#=1 Else wade_vol#=0	
End Function 

Function MakeWater(file$,wl)
	water_tex=LoadTexture(file$)
	ScaleTexture water_tex,20,20
	water=CreatePlane()
	EntityTexture water,water_tex
	PositionEntity water,0,wl,0
	EntityAlpha water,.5
End Function

Function MakeTerrain()
	land_tex=LoadTexture( "terrain_tex.jpg" )
	ScaleTexture land_tex,10,10
	land=LoadTerrain( "terrain_hmap.jpg" )
	ScaleEntity land,8,100,8
	PositionEntity land,-2048,0,-2048
	EntityTexture land,land_tex
	TerrainDetail land,1000,True
End Function

Function MakeSkybox()
	skybox=CreateMesh()
	;front face
	b=LoadBrush( "front.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,-1,+1,-1,0,0:AddVertex s,+1,+1,-1,1,0
	AddVertex s,+1,-1,-1,1,1:AddVertex s,-1,-1,-1,0,1
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	;left face
	b=LoadBrush( "left.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,+1,+1,-1,0,0:AddVertex s,+1,+1,+1,1,0
	AddVertex s,+1,-1,+1,1,1:AddVertex s,+1,-1,-1,0,1
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	;back face
	b=LoadBrush( "back.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,+1,+1,+1,0,0:AddVertex s,-1,+1,+1,1,0
	AddVertex s,-1,-1,+1,1,1:AddVertex s,+1,-1,+1,0,1
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	;right face
	b=LoadBrush( "right.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,-1,+1,+1,0,0:AddVertex s,-1,+1,-1,1,0
	AddVertex s,-1,-1,-1,1,1:AddVertex s,-1,-1,+1,0,1
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	;top face
	b=LoadBrush( "up.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,-1,+1,+1,0,1:AddVertex s,+1,+1,+1,0,0
	AddVertex s,+1,+1,-1,1,0:AddVertex s,-1,+1,-1,1,1
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	;bottom face	
	b=LoadBrush( "down.jpg",49 )
	s=CreateSurface( skybox,b )
	AddVertex s,-1,-1,-1,1,0:AddVertex s,+1,-1,-1,1,1
	AddVertex s,+1,-1,+1,0,1:AddVertex s,-1,-1,+1,0,0
	AddTriangle s,0,1,2:AddTriangle s,0,2,3
	FreeBrush b
	ScaleMesh skybox,100,100,100
	FlipMesh skybox
	EntityFX skybox,1
	EntityOrder skybox,10
End Function